/*!
 *****************************************************************************
 * @file:   ADuCM410_Setup.c
 * @brief:  setup for ADuCM410
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2013-2018 Analog Devices, Inc. All Rights Reserved.

This software is proprietary to Analog Devices, Inc. and its licensors.
By using this software you agree to the terms of the associated
Analog Devices Software License Agreement.
 ******************************************************************************/

#include "SpiLib.h"
#include "ClkLib.h"
#include "MdLib.h"

/*
   User can change following cofiguraion according to application
*/
CLK_SETUP_t gClkSetup = 
{
   //--------- configuration ---------
   .pllEn = 1,
   .rootClkMux = ENUM_CLOCK_CLKCON0_CLKMUX_SPLL,
   .analogClkMux = ENUM_CLOCK_CLKCON0_ANACLKMUX_HFOSC,
    

//-----------------Clock Divider -------------   
   .hclkDiv = ENUM_CLOCK_CLKCON1_CDHCLK_DIV4,
   .pclk0Div = ENUM_CLOCK_CLKCON1_CDPCLK0_DIV4,
   .pclk1Div = ENUM_CLOCK_CLKCON1_CDPCLK1_DIV4,
    
   .clockOff = 0x00000000,  //all clock on

   //---------Interrupt------------
   .pllIntEn = 0,    //0 - disalbe interrrupt related to PLL

   //-------TEST CLOCK -------
   .clkOutput = ENUM_CLOCK_CLKCON0_CLKOUT_B1_PCLK,
};

SPI_SETUP_t gSpi0Setup = 
{
   .spiEn = 1,
   .masterEn = 1,
   .cpha = 1,   //clock pahse
   .cpol = 1,      //clock polarity 
   .wireOrEn = 1,   //1 - SPI wire or mode 
   .lsbFirst = 0,   //0 - MSB transfered first
   .txTrigTransfer = 1, //0 - initiate transfer when read Rx reigster.
   .rxOverwrite = 1,//1 - Rx resigter is overwriten by new serial byte received
   .fifoEmtpyTx = FIFOEMPTY_TX_0,   //1 - send '0' when no valid data in Tx FIFO
   .misoEn = 1,  //1 - enable miso drive in slave mode
   .continousTx = 1,   //1 - transfer continous until no valid data in Tx register.
   .loopback = 0,   //1 - enable loopback
   .threePinMode = 1,  //1 - enable three pin mode, MOSI works as data line

   //----------- Interrupt -------------
   .txrxIrqByte = ENUM_SPI_IEN_IRQMODE_TX1RX1,//Interrupt Occurs After 4 Byte is Transfered or Received
   .csIntEn = 0,
   .txUnderflowIntEn = 0,
   .rxOverflowIntEn = 0,
   .readyIntEn = 0,
   .txDoneIntEn = 0,
   .frameEndIntEn = 0,
   .txFifoEmptyIntEn = 0,

   //------------- Dma -----------------
   .txDmaEn = 0,
   .rxDmaEn = 0,

   //-------------- Read command mode -----------
   .readCmdEn = 0, //0 - disable read command mode
   .txByte = 0,    //number of bytes to be transmitted before reading data
   .txRxOverlap = 0,
};

MDIO_SETUP_t gMdioSetup = 
{
   .mdEn = 1,                                        //MDIO Enable
   .mdDrive = ENUM_MDIO_MDCON_MD_DRV_MD_DRV_OD,        //Enable Open-drain of MDIO Drive
   .mdPhyWidth = ENUM_MDIO_MDCON_MD_PHM_EN_5BIT,    // PHY Address Bit Width = 5bits
   .mdDevAddr = 0x01,                               //device address. Normally 01. 
   .mdPhySel = 0x1F,          //Selects Expected PHYADR Bits. All the 5 bits PHYADR according to correspond software provided
   .mdSwAddr = 0x05,           //Software Provided PHYADR. 
   .mdPaddrPe= 0x1F,       //PADDR[4:0] PE Control.default 
   
//-----------Interrupt ------------     
   .mdPhyNonMatchIen = 0,  //Interrupt Enable for MD_PHYN. Disabled
   .mdPhyMatchIen = 1,     //Interrupt Enable for MD_PHYM. Disabled
   .mdDevNonMatchIen = 0,  //Interrupt Enable for MD_DEVN. Disabled
   .mdDevMatchIen = 1,         //Interrupt Enable for MD_DEVM. Disabled
   .mdReadIen = 1,              //Interrupt Enable for MD_RDF.
   .mdIncReadIen= 1,        //Interrupt Enable for MD_INCF.
   .mdAddressIen = 1,            //Interrupt Enable for MD_ADRF.
   .mdWriteIen = 1,              //Interrupt Enable for MD_WRF.
   
};


