/*!
 *****************************************************************************
 * @file:
 * @brief:
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2013-2018 Analog Devices, Inc. All Rights Reserved.

This software is proprietary to Analog Devices, Inc. and its licensors.
By using this software you agree to the terms of the associated
Analog Devices Software License Agreement.
 ******************************************************************************/

#include "ADuCM410.h"
#include "WdtLib.h"
#include "common.h"
#include "DioLib.h"
#include "ClkLib.h"
#include "SpiLib.h"
#include "MdLib.h"

typedef struct
{
   uint32_t op;
   uint16_t data;
	 uint16_t padding;
}MDIO_FRAME_Type;

#define MDIO_OP_ADDRESS 0
#define MDIO_OP_WRITE 1
#define MDIO_OP_READ 3
#define MDIO_OP_POST_READ_INC 2


extern uint16_t mdioAddData;
extern MDIO_FRAME_Type mdioFrame;
#define MDIO_TX_BUF_LENGTH 4
extern uint16_t mdioTxBuf[MDIO_TX_BUF_LENGTH];
extern volatile uint32_t txCnt;

uint16_t mdioAddData;
MDIO_FRAME_Type mdioFrame;
uint16_t mdioTxBuf[MDIO_TX_BUF_LENGTH]={0x4144,0x7543,0x4D34,0x3130};
volatile uint32_t txCnt;

// Function Definitions
void delay(uint32_t t);
void Spi0Init(void);
void MdioInit(void);
uint8_t MdFrame(uint32_t OP, uint32_t phyAddr, uint32_t devAddr, uint16_t *pAddrData);

#define TEST_LOOP_COUNT 1

int main (void)
{
    uint32_t status;
    uint32_t ta;
    txCnt = 0;
    WdtGo(false);
    
    ClkSetup(&gClkSetup);

    SystemInit();
    INIT();
    SafetyWait();

    DEBUG_MESSAGE("Test Started");

    MdioInit();    
    Spi0Init();
    
    for(uint32_t loop=0;loop<TEST_LOOP_COUNT;loop++)
    {
       DEBUG_MESSAGE("test cycle: %d",loop);
       //----- test address frame
       mdioAddData = 0x55AA;
       ta = MdFrame(MDIO_OP_ADDRESS,gMdioSetup.mdSwAddr, gMdioSetup.mdDevAddr, &mdioAddData);
       do
       {
          status = (mdioFrame.op==MDIO_OP_ADDRESS) && (mdioFrame.data==mdioAddData);
       }
       while(status == 0); //wait for correct transmission
       DEBUG_MESSAGE("master address : 0x%x, TA is %d",mdioFrame.data, ta);   //note ADuCM410 MDIO slave doesn't drive TA in address frame
       
       //----- test write frame
       mdioAddData = 0x7A7E;
       ta = MdFrame(MDIO_OP_WRITE,gMdioSetup.mdSwAddr, gMdioSetup.mdDevAddr, &mdioAddData);
       do
       {
          status = (mdioFrame.op==MDIO_OP_WRITE);
       }
       while(status == 0); //wait for correct transmission
       DEBUG_MESSAGE("master write data : 0x%x, TA is %d",mdioFrame.data,ta);//note ADuCM410 MDIO slave doesn't drive TA in write frame
       
       //----- test post_read_add_inc frame
       mdioAddData = 0;
       MdTxd(mdioTxBuf[txCnt]);
       ta = MdFrame(MDIO_OP_POST_READ_INC,gMdioSetup.mdSwAddr, gMdioSetup.mdDevAddr, &mdioAddData);
       do
       {
          status = (mdioFrame.op==MDIO_OP_POST_READ_INC);
       }
       while(status == 0); //wait for correct transmission
       DEBUG_MESSAGE("master Inc read data: 0x%x, TA is %d",mdioAddData,ta);
       
       //----- test read frame
       mdioAddData = 0;
       MdTxd(mdioTxBuf[txCnt]);
       ta = MdFrame(MDIO_OP_READ,gMdioSetup.mdSwAddr, gMdioSetup.mdDevAddr, &mdioAddData);
       do
       {
          status =(mdioFrame.op==MDIO_OP_READ);
       }
       while(status == 0); //wait for correct transmission
       DEBUG_MESSAGE("master read data: 0x%x, TA is %d",mdioAddData,ta);
       
    }

    while(1)
    {
        delay(10000);
        //WdtRefresh();
    }

}

void delay(uint32_t t)
{
   uint32_t i;
   for(i=0;i<t;i++);
}

void Spi0Init()
{
   //GPIO Mux
   DioCfgPin(pADI_GPIO0,PIN0,P0_0_SPI0_SCLK);
   DioCfgPin(pADI_GPIO0,PIN2,P0_2_SPI0_MOSI);
   DioPwrCfgPin(pADI_GPIO0,PIN0,GPIO_PWR_1VX);  //use dvdd1 for high voltage level
   DioPwrCfgPin(pADI_GPIO0,PIN2,GPIO_PWR_1VX);
   DioDsPin(pADI_GPIO0,PIN0,1);
   DioDsPin(pADI_GPIO0,PIN2,1);
   
   
   DioPullCfgPin(pADI_GPIO0,PIN2,1);
   DioPullCfgPin(pADI_GPIO0,PIN0,1);
   DioPulPin(pADI_GPIO0,PIN0,1);
   DioPulPin(pADI_GPIO0,PIN2,1);
   
   //SPI setup
   SpiSetup(pADI_SPI0,&gSpi0Setup);
   //set SPI clock
   SpiBaud(pADI_SPI0,49);   
}

void MdioInit()
{
   //GPIO mux
   DioCfgPin(pADI_GPIO3,PIN5,P3_5_MCK);
   DioCfgPin(pADI_GPIO3,PIN6,P3_6_MDIO);
   //use software address, commentted below
   //DioCfgPin(pADI_GPIO3,PIN0,P3_0_MDADDR0);
   //DioCfgPin(pADI_GPIO3,PIN1,P3_1_MDADDR1);
   //DioCfgPin(pADI_GPIO3,PIN2,P3_2_MDADDR2);
   //DioCfgPin(pADI_GPIO3,PIN3,P3_3_MDADDR3);
   //DioCfgPin(pADI_GPIO3,PIN4,P3_4_MDADDR4);
   DioDsPin(pADI_GPIO3,PIN6,3);
   DioDsPin(pADI_GPIO3,PIN5,3);
   
   DioPullCfgPin(pADI_GPIO3,PIN6,1);
   DioPullCfgPin(pADI_GPIO3,PIN5,1);
   DioPulPin(pADI_GPIO3,PIN6,1);
   DioPulPin(pADI_GPIO3,PIN5,1);
   
   MdSetup(&gMdioSetup);
   NVIC_EnableIRQ(MDIO_IRQn);  //Enable MDIO interrupt source in NVIC.
}

/*
@brief uint8_t MdFrame(uint32_t OP, uint32_t phyAddr, uint32_t devAddr, uint16_t *pAddrData)
      ===========send MDIO frame
 
@param OP: {MDIO_OP_ADDRESS,MDIO_OP_WRITE,MDIO_OP_READ,MDIO_OP_POST_READ_INC}
@param phyAddr:
    5 bits phy adress
@param devAddr:
    5 bits device address
@param pAddrData:
    pointer to address or data that send/receive to/from MDIO slave
@return monitored TA value on MDIO
**/
uint8_t MdFrame(uint32_t OP, uint32_t phyAddr, uint32_t devAddr, uint16_t *pAddrData)
{
   uint32_t i=0;
   uint8_t TA=3;
   uint8_t tempByte;
   uint16_t addrData = *pAddrData;
   uint8_t dummyDat;
   static uint8_t monitorData[8];
   uint32_t index = 0;
   
   pADI_SPI0->CTL |= BITM_SPI_CTL_RFLUSH;
   pADI_SPI0->CTL &=  ~(BITM_SPI_CTL_TIM|BITM_SPI_CTL_RFLUSH);
   pADI_SPI0->CNT = 8; 
   
   if(OP >= MDIO_OP_POST_READ_INC)
   { 
      //enable overlap to monitor MOSI
      pADI_SPI0->RDCTL = BITM_SPI_RDCTL_CMDEN|BITM_SPI_RDCTL_THREEPIN|   \
         (5<<BITP_SPI_RDCTL_TXBYTES)|BITM_SPI_RDCTL_OVERLAP;  //SEND 6 BYTES BEFORE reading 
   }
   else
   {
      //enable overlap to monitor MOSI
      pADI_SPI0->RDCTL = BITM_SPI_RDCTL_CMDEN|BITM_SPI_RDCTL_THREEPIN|   \
         (7<<BITP_SPI_RDCTL_TXBYTES)|BITM_SPI_RDCTL_OVERLAP;  //SEND 8 BYTES BEFORE reading 
   }  
   
   for(i=0;i<4;i++)
   {
      pADI_SPI0->TX = 0xFF;   //send 32bit PREAMBLE
   }
   tempByte = (uint8_t)(0x3F& ((OP<<4)|(phyAddr>>1)));    //ST included 
   pADI_SPI0->TX = tempByte;
   tempByte = (uint8_t)((phyAddr<<7)|(devAddr<<2)|TA);
   pADI_SPI0->TX = tempByte;
   
   dummyDat = (uint8_t)(pADI_SPI0->RX);   //dummy read to start transfer
   (void)dummyDat;
   
   switch(OP)
   {
   case MDIO_OP_ADDRESS: 
      pADI_SPI0->TX = addrData>>8;
      pADI_SPI0->TX = (uint8_t)addrData; 
      while((pADI_SPI0->FIFOSTAT>>BITP_SPI_FIFOSTAT_RX) < 8);
      for(index=0;index<8;index++)
      {
         monitorData[index] = (uint8_t)pADI_SPI0->RX;
      }
      break;
   case MDIO_OP_WRITE:
      pADI_SPI0->TX = addrData>>8;
      pADI_SPI0->TX = (uint8_t)addrData; 
      while((pADI_SPI0->FIFOSTAT>>BITP_SPI_FIFOSTAT_RX) < 8);
      for(index=0;index<8;index++)
      {
         monitorData[index] = (uint8_t)pADI_SPI0->RX;
      }
      break;
   case MDIO_OP_READ:
      while((pADI_SPI0->FIFOSTAT>>BITP_SPI_FIFOSTAT_RX) < 8);
      for(index=0;index<8;index++)
      {
         monitorData[index] = (uint8_t)pADI_SPI0->RX;
         delay(160);
      }
      *pAddrData = (uint16_t)((monitorData[6]<<8)|monitorData[7]);
      break;
   case MDIO_OP_POST_READ_INC:
      while((pADI_SPI0->FIFOSTAT>>BITP_SPI_FIFOSTAT_RX) < 8);
      for(index=0;index<8;index++)
      {
         monitorData[index] = (uint8_t)pADI_SPI0->RX;
      }
      *pAddrData = (uint16_t)((monitorData[6]<<8)|monitorData[7]);
      break;
   default:
      break;
   }

   return (0x3&monitorData[5]); //return monitored TA bits
}

void MDIO_Int_Handler()
{
   uint32_t iMdioSta;
   iMdioSta = pADI_MDIO->MDSTA;	
   if(iMdioSta & BITM_MDIO_MDSTA_MD_WRF)                //if Write Frame Satus
   {
      mdioFrame.op = MDIO_OP_WRITE;
      mdioFrame.data = pADI_MDIO->MDRXD;
   } 
   if(iMdioSta & BITM_MDIO_MDSTA_MD_ADRF)                //if Address Frame Status
   {
      mdioFrame.op = MDIO_OP_ADDRESS;
      mdioFrame.data = pADI_MDIO->MDADR;
   }
   if(iMdioSta & BITM_MDIO_MDSTA_MD_INCF)                //if Post Read Increment Address Frame Status
   {
      txCnt++;
      mdioFrame.op = MDIO_OP_POST_READ_INC;
      txCnt = (txCnt>=MDIO_TX_BUF_LENGTH)? 0:txCnt;
   }   
   if(iMdioSta & BITM_MDIO_MDSTA_MD_RDF )            //if Read Frame Status
   {
      mdioFrame.op = MDIO_OP_READ;
   }   
   if(iMdioSta & BITM_MDIO_MDSTA_MD_DEVM)
   {
     //if Device Address Matching Status
      DEBUG_PIN0_TOGGLE();
   }
   if(iMdioSta & BITM_MDIO_MDSTA_MD_DEVN)
   {
      //if Device Address None Match Status
      DEBUG_PIN0_TOGGLE();
   }
   if(iMdioSta & BITM_MDIO_MDSTA_MD_PHYM)
   {
     //if PHY Address Matching Status
      DEBUG_PIN0_TOGGLE();
   }
   if(iMdioSta & BITM_MDIO_MDSTA_MD_PHYN)
   {
       //if PHY Address Non Matching Status
      DEBUG_PIN0_TOGGLE();
   } 

}




